<?php

namespace OrganizationsManagement\Service;

use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Mail\SmtpCore;
use OrganizationsManagement\Util\Utilities;

class SendAlertSla
{

    private $arrayEmail = array();
    const CDGRUPFASE_IMPLEMENTATION = 30;
    const CDGRUPFASE_SPECIFICATION = 13;
    const IDANTES = 'A';
    const IDDEPOIS = 'D';
    const RESP_PRODUCT = 'P';
    const RESP_STRUCTURE = 'E';
    const MAIL_FOOTER_MESSAGE               = "Email automático enviado pelo Organization Management.";
    public function sendAlertSla()
    {
        header('Content-type: application/json; charset=utf-8');
        $arrayCallAlerts = InstanceProvider::getRepository()->getSendAlertSla()->getCallAlerts();
        $arraySendEmail = array();
        $countCallsWithoutResp = 0;
        foreach ($arrayCallAlerts as $callAlert) {
            $arrayAlert = array();

            $arrayAlert = $this->setInfoEmailSlaAlert($callAlert, $arrayAlert);

            $emailResp = $this->getEmailresponsble($callAlert, $arrayAlert['NRCALL']);

            if (empty($emailResp)) {
                $countCallsWithoutResp++;
                echo('Não foi encontrado os responsalveis pelo chamado: "' . $arrayAlert['NRCALL'] . '"' . "\n");
            }
            foreach ($emailResp as $email) {
                if (substr($email, -1) == '!') {
                    echo($email . ' do chamado: "' . $arrayAlert['NRCALL'] . '"' . "\n");
                } else {
                    if (empty($arraySendEmail[$email])) {
                        $arraySendEmail[$email] = array();
                    }
                    $arraySendEmail[$email][] = $arrayAlert;
                }
            }
        }
        foreach ($arraySendEmail as $email => $infoSlaAlert) {

            try{
                $this->sendErrorEmail($email,$infoSlaAlert);
                echo( 'Foi enviado um email de SLA alerta  para '.$email.' contendo '.count($infoSlaAlert).' chamados'."\n");
            }catch (\Exception $e){
                echo( 'Erro ao enviar email de SLA alerta  para '.$email.' contendo '.count($infoSlaAlert).' chamados'."\n");
                echo( 'Erro: '.$e->getMessage()."\n");
            }
        }
    }

    private function sendErrorEmail($email,$rows)
    {
        $template = $this->getTemplateEmailSlaEmail();
        $templateRow = $this->getTemplateEmailSlaEmailRow();
        $assunto = "Alerta de chamados SLA";
        $tab= '';
        foreach ($rows as $row) {
            $tab .= $this->traduzConteudoTemplateRow($templateRow, $row);
        }
        $conteudo = $this->traduzConteudoTemplate($template, 'Os seguintes chamados devem ser priorizados em relação a SLA',$tab);
        $smtpMail = new SmtpCore();
        $smtpMail->enviaHtml($email, 'om.sla@teknisa.com', $assunto, $conteudo);

    }
    private function traduzConteudoTemplate($template, $actionDescription,$tab){
        $conteudoTraduzido = file_get_contents($template);
        $conteudoTraduzido = str_replace('%actionDescription%', $actionDescription, $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%tabela%', $tab, $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%footerMessage%', self::MAIL_FOOTER_MESSAGE, $conteudoTraduzido);
        return $conteudoTraduzido;
    }
    private function traduzConteudoTemplateRow($template, $row){
        $conteudoTraduzido = file_get_contents($template);
        $conteudoTraduzido = str_replace('%id%', $row['NRCALL'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%name%', $row['NAME'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%product%', $row['PRODUCT_NAME'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%client%', $row['NAME_CLIENTE'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%typeAlert%', $row['TYPE_ALERT_NAME'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%timeContract%', $row['QT_SLA_CONTRATO'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%classification%', $row['LABEL_CLASSIFICATION'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%timeBefore%', $row['TIME_BEFORE'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%timeAfter%', $row['TIME_AFTER'], $conteudoTraduzido);
        $conteudoTraduzido = str_replace('%totalTime%', $row['TOTAL_TIME'], $conteudoTraduzido);
        return $conteudoTraduzido;
    }

    private function getTemplateEmailSlaEmail(){
        return __DIR__.'/../Util/EmailTemplates/sla_alert.html';
    }

    private function getTemplateEmailSlaEmailRow(){
        return __DIR__.'/../Util/EmailTemplates/sla_alert_row.html';
    }

    /**
     * @param $callAlert
     * @param array $arrayAlert
     * @return array
     */
    protected function setInfoEmailSlaAlert($callAlert, array $arrayAlert)
    {
        $arrayAlert['NRCALL'] = $callAlert['ID_CALL'];
        $arrayAlert['NAME'] = $callAlert['NAME'];
        $arrayAlert['LABEL_CLASSIFICATION'] = $callAlert['LABEL_CLASSIFICATION'];
        $arrayAlert['NAME_CLIENTE'] = $callAlert['CLIENT_NAME'];
        $arrayAlert['TOTAL_TIME'] = $callAlert['QTSLA_ESPECIFICACAO'];
        $arrayAlert['QT_SLA_CONTRATO'] = $callAlert['QT_SLA_CONTRATO'];
        $arrayAlert['PRODUCT_NAME'] = $callAlert['PRODUCT_NAME'];
        $arrayAlert['TYPE_ALERT_NAME'] = 'Especificação';
        if ($callAlert['CDGRUPFASE'] == self::CDGRUPFASE_IMPLEMENTATION) {
            $arrayAlert['TOTAL_TIME'] = $callAlert['QTSLA_TOT_TEK'];
            $arrayAlert['TYPE_ALERT_NAME'] = 'Implementação';
        }


        if ($callAlert['IDANTESDEPOIS'] == self::IDANTES) {
            $arrayAlert['TIME_BEFORE'] = $callAlert["QT_SLA_CONTRATO"] - $arrayAlert['TOTAL_TIME'];
            $arrayAlert['TIME_AFTER'] = null;
        } else {
            $arrayAlert['TIME_AFTER'] = $arrayAlert['TOTAL_TIME'] - $callAlert["QT_SLA_CONTRATO"];
            $arrayAlert['TIME_BEFORE'] = null;
        }
        return $arrayAlert;
    }

    /**
     * @param $callAlert
     * @param $nrcall
     * @return array
     */
    protected function getEmailresponsble($callAlert, $nrcall)
    {
        $struEmail = $callAlert['STRUCTURE_EMAIL'];
        $prodEmail = $callAlert['PRODUCT_EMAIL'];

        if ($callAlert['ID_RESP_PROD_ESTRUT'] = self::RESP_STRUCTURE) {
            $emailResp = $this->getEmail($nrcall, $struEmail, $prodEmail);
        } else {
            $emailResp = $this->getEmail($nrcall, $prodEmail, $struEmail);
        }
        return $emailResp;
    }

    /**
     * @param $nrcall
     * @param $prinEmail
     * @param $seguEmail
     * @return array
     */
    protected function getEmail($nrcall, $prinEmail, $seguEmail)
    {
        $emailResp = array();
        if (!empty($prinEmail)) {
            if (substr($prinEmail, -1) == '!') {
                echo($prinEmail . ' do chamado: "' . $nrcall . '"' . "\n");
                $emailResp = Utilities::getArrayByString($seguEmail);
            } else {
                $emailResp = Utilities::getArrayByString($prinEmail);
            }
        } else if (!empty($seguEmail)) {
            $emailResp = Utilities::getArrayByString($seguEmail);
        }
        return $emailResp;
    }
}
