function SetFormsService(templateManager, localStorageEngine, ScreenService, restEngine) {


    this.saveRow = function(args, closeSwipe) {
        args.owner.widget.saveRow().then(function() {
            var currentRow = args.owner.widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            args
                .owner
                .widget
                .reload();
            if(closeSwipe) {
                ScreenService.closeSwipe();
            }
        });
    };

    this.openSelectionOnce = function(name) {
        var filterWidget = templateManager.container.getWidget(name + "_FILTER");
        if(!filterWidget.wasOpened){
            filterWidget.edit();
            ScreenService.openPopup(filterWidget);
            filterWidget.wasOpened = true;
        }
    };
    this.openSelection = function(name) {
        var filterWidget = templateManager.container.getWidget(name + "_FILTER");
        filterWidget.edit();
        ScreenService.openPopup(filterWidget);
    };
    this.getContainerName = function() {
        return templateManager.container.name;
    };

    this.backOrCancel = function(args) {
        if (args.row.__is_new) {
            args.owner.widget.list(true);
            args.owner.widget.dataSource.data.pop();
            args.owner.widget.currentRow = {};
            ScreenService.goBack();
        } else {
            args.owner.widget.view();
            for (var i in args.owner.widget.backupRow) {
                args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
            }
        }
    };

    this.edit = function(widget) {
        widget.backupRow = angular.copy(widget.currentRow);
        widget.edit();
    };

    this.setContextVar = function(name, containerName, value) {
        localStorageEngine.setLocalVar(containerName + '_' + name, value);
    };

    this.getContextVar = function(name, containerName) {
        return localStorageEngine.getLocalVar(containerName + '_' + name);
    };

    var _getWidget = function(name) {
        var widget = templateManager.container.getWidget(widgetName);
        if (widget) {
            return widget;
        }
        throw "Widget not found";
    };

    var _getField = function(name, widgetName) {
        var widget = _getWidget(widgetName);
        var field = widget.getField(name);
        if (field) {
            return field;
        }
        throw "Field not found";
    };

    var _getAction = function(name, widgetName) {
        var widget = _getWidget(widgetName);
        var action = widget.getAction(name);
        if (action) {
            return action;
        }
        throw "Action not found";
    };

    this.getFieldValue = function(name, widgetName) {
        var widget = _getWidget(widgetName);
        if (widget && widget.currentRow) {
            return widget.currentRow[name];
        } else {
            return "";
        }
    };

    this.setEnableField = function(name, widgetName) {
        _getField(name, widgetName).readOnly = false;
    };

    this.setDisableField = function(name, widgetName) {
        _getField(name, widgetName).readOnly = true;
    };

    this.setVisibleField = function(name, widgetName) {
        _getField(name, widgetName).isVisible = true;
    };

    this.setInvisibleField = function(name, widgetName) {
        _getField(name, widgetName).isVisible = false;
    };

    this.setEnableAction = function(name, widgetName) {
        _getAction(name, widgetName).readOnly = false;
    };

    this.setDisableAction = function(name, widgetName) {
        _getAction(name, widgetName).readOnly = true;
    };

    this.setVisibleAction = function(name, widgetName) {
        _getAction(name, widgetName).isVisible = true;
    };

    this.setInvisibleAction = function(name, widgetName) {
        _getAction(name, widgetName).isVisible = false;
    };

    this.now = function() {
        return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
    };

    this.today = function() {
        return moment(new Date()).format('DD/MM/YYYY');
    };

    this.openReport = function(route, reportName, params) {
        params.__report_name = reportName;
        return restEngine.post(route, {
            "requestType": "Row",
            "row": params
        }, null, null, null, null, 3000000).then(function(response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                window.open(report.message);
            }
        }).catch(function(response) {
            if (response.data && response.data.error) {
                ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
            }
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('SetFormsService', SetFormsService);
});